@@ -22,7 +22,8 @@ from python_kemptech_api.exceptions import (
     SubVsCannotCreateSubVs,
     BackupFailed,
     DownloadUserCertFailed,
-    UnauthorizedAccessError, UserAlreadyExistsException)
+    UnauthorizedAccessError,
+    UserAlreadyExistsException)
 from python_kemptech_api.capabilities import CAPABILITIES, DEFAULT
 from .utils import (
     validate_port,
@@ -46,11 +47,14 @@ class HttpClient(object):
     def __init__(self, tls_version=utils.DEFAULT_TLS_VERSION, cert=None):
         self._tls_version = tls_version
         self.cert = cert
+        self._tls_session = Session()
+        self._tls_session.mount("http://", UseTlsAdapter(self._tls_version))
 
     def __enter__(self):
         return self
 
     def __exit__(self, exc_type, exc_val, exc_tb):
+        self._tls_session.close()
         return False
 
     def _do_request(self, http_method, rest_command,
@@ -70,9 +74,6 @@ class HttpClient(object):
                                             cmd=rest_command)
         log.debug("Request is: %s", cmd_url)
 
-        self._tls_session = Session()
-        self._tls_session.mount("http://", UseTlsAdapter(self._tls_version))
-
         try:
             if file is not None:
                 with open(file, 'rb') as payload:
@@ -89,6 +90,7 @@ class HttpClient(object):
                                                      verify=False,
                                                      headers=headers,
                                                      cert=self.cert)
+            self._tls_session.close()
             if response.status_code == 401:
                 log.warning("Cannot authenticate to %s check that the "
                             "credentials are correct.", self.ip_address)
@@ -369,15 +371,16 @@ class LoadMaster(HttpClient):
                                                            ip=self.ip_address,
                                                            port=self.port)
         try:
-            self._tls_session = Session()
-            self._tls_session.mount("http://",
-                                    UseTlsAdapter(self._tls_version))
+            #self._tls_session = Session()
+            #self._tls_session.mount("http://",
+                                    #UseTlsAdapter(self._tls_version))
             if self.cert:
                 self._tls_session.get(url, verify=False, timeout=1,
                                       cert=self.cert)
             else:
                 self._tls_session.get(url, verify=False, timeout=1,
                                       cert=self.cert)
+            self._tls_session.close()
             return True
         except exceptions.RequestException:
             return False
@@ -629,7 +632,7 @@ class VirtualService(HttpClient, KempBaseObjectModel):
             self.ip_address = loadmaster_info["ip_address"]
         except KeyError:
             raise VirtualServiceMissingLoadmasterInfo("ip_address")
-        super(VirtualService, self).__init__()
+        super(VirtualService, self).__init__(cert=self.cert)
 
     def __str__(self):
         try:
@@ -678,30 +681,37 @@ class VirtualService(HttpClient, KempBaseObjectModel):
             }
 
     def save(self, update=False):
-        if not update:
-            if self._is_sub_vs:
-                # Hell, thy name be subvs
-                response = self._get("/showvs", self._get_base_parameters())
-                data = get_data(response)
-                existing_subvs_entries = get_sub_vs_list_from_data(data)[0]
-                params = self._get_base_parameters()
-                params["createsubvs"] = ""
-                response = self._get("/modvs", params)
-                data = get_data(response)
-                new_subvs_entries, subvs_data = get_sub_vs_list_from_data(data)
-                s = set(existing_subvs_entries)
-                created_subvs_id = [x for x in new_subvs_entries if x not in s]
-                newly_created_vs_params = {"vs": created_subvs_id}
-                self.subvs_data = subvs_data[created_subvs_id[0]]
-                self.subvs_data['parentvs'] = self.vs
-                response = self._get("/showvs", newly_created_vs_params)
+        try:
+            if not update:
+                if self._is_sub_vs:
+                    # Hell, thy name be subvs
+                    response = self._get("/showvs",
+                                         self._get_base_parameters())
+                    data = get_data(response)
+                    existing_subvs_entries = get_sub_vs_list_from_data(data)[0]
+                    params = self._get_base_parameters()
+                    params["createsubvs"] = ""
+                    response = self._get("/modvs", params)
+                    data = get_data(response)
+                    new_subvs_entries, subvs_data = \
+                        get_sub_vs_list_from_data(data)
+                    s = set(existing_subvs_entries)
+                    created_subvs_id = [x
+                                        for x in new_subvs_entries
+                                        if x not in s]
+                    newly_created_vs_params = {"vs": created_subvs_id}
+                    self.subvs_data = subvs_data[created_subvs_id[0]]
+                    self.subvs_data['parentvs'] = self.vs
+                    response = self._get("/showvs", newly_created_vs_params)
+                else:
+                    response = self._get("/addvs", self.to_api_dict())
             else:
-                response = self._get("/addvs", self.to_api_dict())
-        else:
-            if self._is_sub_vs:
-                # Update the underlying "Rs" part of the subvs as well
-                self._get("/modrs", self._subvs_to_dict())
-            response = self._get("/modvs", self.to_api_dict())
+                if self._is_sub_vs:
+                    # Update the underlying "Rs" part of the subvs as well
+                    self._get("/modrs", self._subvs_to_dict())
+                response = self._get("/modvs", self.to_api_dict())
+        except KempTechApiException:
+            raise
 
         if is_successful(response):
             vs_data = get_data(response)
@@ -825,7 +835,7 @@ class RealServer(HttpClient, KempBaseObjectModel):
         except KeyError:
             raise RealServerMissingLoadmasterInfo("ip_address")
 
-        super(RealServer, self).__init__()
+        super(RealServer, self).__init__(cert=self.cert)
 
     def __str__(self):
         return 'Real Server {} on {}'.format(self.rs, self.vs)
