@@ -323,7 +323,7 @@ class TestLmApiWrapperCalls:
         params = {'user': user}
         self.lm._get.assert_called_with('/userdellocal', params)
 
-    def test_set_user_perms(self):
+    def test_set_user_perms_as_list(self):
         user = 'shane'
         perms = ['root', 'vs']
         self.lm.set_user_perms(user, perms)
@@ -333,6 +333,16 @@ class TestLmApiWrapperCalls:
         }
         self.lm._get.assert_called_with('/usersetperms', params)
 
+    def test_set_user_perms_as_string(self):
+        user = 'shane'
+        perms = 'root'
+        self.lm.set_user_perms(user, perms)
+        params = {
+            'user': user,
+            'perms': 'root',
+        }
+        self.lm._get.assert_called_with('/usersetperms', params)
+
     def test_new_user_cert(self):
         user = 'shane'
         perms = ['root', 'vs']
